/**************************************************************************//**
*   @file   ad5172.c
*   @brief  Implementation of AD5172 Driver for Microblaze processor.
*   @author DBogdan (dragos.bogdan@analog.com)
*
*******************************************************************************
* Copyright 2011(c) Analog Devices, Inc.
*
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*  - Redistributions of source code must retain the above copyright
*    notice, this list of conditions and the following disclaimer.
*  - Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in
*    the documentation and/or other materials provided with the
*    distribution.
*  - Neither the name of Analog Devices, Inc. nor the names of its
*    contributors may be used to endorse or promote products derived
*    from this software without specific prior written permission.
*  - The use of this software may or may not infringe the patent rights
*    of one or more patent holders.  This license does not release you
*    from the requirement that you obtain separate licenses from these
*    patent holders to use this software.
*  - Use of the software either in source or binary form, must be run
*    on or directly connected to an Analog Devices Inc. component.
*
* THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR IMPLIED
* WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT, MERCHANTABILITY
* AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
* INTELLECTUAL PROPERTY RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
* ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*******************************************************************************
*   SVN Revision: 468
******************************************************************************/
#define AD5172_GLOBALS

/*****************************************************************************/
/* Include Files                                                             */
/*****************************************************************************/
#include "ad5172.h"
#include "../drivers/i2c.h"
#include "../drivers/gpio.h"

/****************************************************************************/
/*  Private variables                                                       */
/****************************************************************************/
static unsigned char    serialWord          = 0;    // The 8 bits word which will be sent to the AD5172 circuit
                                                    // as instruction and data.
static unsigned char    shutdown1ButtonOld  = 0;    // Old value of variable shutdown1Button.
static unsigned char    shutdown2ButtonOld  = 0;    // Old value of variable shutdown2Button.

/*****************************************************************************/
/*  Aliases for the uC Probe data                                            */
/*****************************************************************************/
volatile unsigned char* rdac1Value      = &AD5172_data[0].uchData;  // Value of the DAC 0 register.
volatile unsigned char* rdac2Value      = &AD5172_data[1].uchData;  // Value of the DAC 1 register.
volatile unsigned char* loadRdac1Button = &AD5172_data[2].uchData;  // Position of the Load RDAC1 button.
volatile unsigned char* loadRdac2Button = &AD5172_data[3].uchData;  // Position of the Load RDAC2 button.
volatile unsigned char* a0              = &AD5172_data[4].uchData;  // Instruction bit A0.
volatile unsigned char* sdBit           = &AD5172_data[5].uchData;  // Instruction bit SD.
volatile unsigned char* tBit            = &AD5172_data[6].uchData;  // Instruction bit T.
volatile unsigned char* zeroBit         = &AD5172_data[7].uchData;  // Instruction bit 0.
volatile unsigned char* owBit           = &AD5172_data[8].uchData;  // Instruction bit OW.
volatile unsigned char* x1Bit           = &AD5172_data[9].uchData;  // Instruction bit X1.
volatile unsigned char* x2Bit           = &AD5172_data[10].uchData; // Instruction bit X2.
volatile unsigned char* x3Bit           = &AD5172_data[11].uchData; // Instruction bit X3.
volatile unsigned char* d7Bit           = &AD5172_data[12].uchData; // Data bit D7.
volatile unsigned char* d6Bit           = &AD5172_data[13].uchData; // Data bit D6.
volatile unsigned char* d5Bit           = &AD5172_data[14].uchData; // Data bit D5.
volatile unsigned char* d4Bit           = &AD5172_data[15].uchData; // Data bit D4.
volatile unsigned char* d3Bit           = &AD5172_data[16].uchData; // Data bit D3.
volatile unsigned char* d2Bit           = &AD5172_data[17].uchData; // Data bit D2.
volatile unsigned char* d1Bit           = &AD5172_data[18].uchData; // Data bit D1.
volatile unsigned char* d0Bit           = &AD5172_data[19].uchData; // Data bit D0.
volatile unsigned char* sendDataButton  = &AD5172_data[20].uchData; // Position of the Send Data button.
volatile unsigned char* bytesButton     = &AD5172_data[21].uchData; // Position of the Bytes to Send button.
volatile unsigned char* shutdown1Button = &AD5172_data[22].uchData; // Position of the Shutdown 1 button.
volatile unsigned char* shutdown2Button = &AD5172_data[23].uchData; // Position of the Shutdown 2 button.
volatile unsigned char* program1Button  = &AD5172_data[24].uchData; // Position of the Program 1 button.
volatile unsigned char* program2Button  = &AD5172_data[25].uchData; // Position of the Program 2 button.
volatile unsigned char* fuse1Status     = &AD5172_data[26].uchData; // Value of the Fuse 1 Status.
volatile unsigned char* fuse2Status     = &AD5172_data[27].uchData; // Value of the Fuse 2 Status.

/**************************************************************************//**
 * @brief Initializes the communication with the device and read fuses status.
 *
 * @param None.
 *
 * @return Always returns TRUE.
******************************************************************************/
int AD5172_Init(void)
{
    char rxBuf[2];

    I2C_Init(XPAR_AXI_IIC_0_BASEADDR, AD5172_I2C_ADDRESS);

    serialWord = 0x00;
    I2C_Write(XPAR_AXI_IIC_0_BASEADDR, AD5172_I2C_ADDRESS, -1, 1, (char*)&serialWord); // Select RDAC1.

    I2C_Read(XPAR_AXI_IIC_0_BASEADDR, AD5172_I2C_ADDRESS, -1, 2, rxBuf); // Read data.
    *fuse1Status = (rxBuf[1] >> 6);

    serialWord = 0x80;
    I2C_Write(XPAR_AXI_IIC_0_BASEADDR, AD5172_I2C_ADDRESS, -1, 1, (char*)&serialWord); // Select RDAC2.

    I2C_Read(XPAR_AXI_IIC_0_BASEADDR, AD5172_I2C_ADDRESS, -1, 2, rxBuf); // Read data.
    *fuse2Status = (rxBuf[1] >> 6);

    return (TRUE);
}

/**************************************************************************//**
 * @brief Update the device's monitored and controlled parameters
 *
 * @param None
 *
 * @return Always returns success
******************************************************************************/
int AD5172_UpdateParameters(void)
{
    char txBuf[2];
    char rxBuf[2];

    if(*loadRdac1Button)    // If the Load RDAC1 button was pressed,
    {                       // loads the value from the Slider 1 to the RDAC1 register.
        I2C_Write(XPAR_AXI_IIC_0_BASEADDR, AD5172_I2C_ADDRESS, 0x00, 1, (char*)rdac1Value);
        *loadRdac1Button = 0;
    }

    if(*loadRdac2Button)    // If the Load RDAC1 button was pressed,
    {                       // loads the value from the Slider 2 to the RDAC1 register.
        I2C_Write(XPAR_AXI_IIC_0_BASEADDR, AD5172_I2C_ADDRESS, 0x80, 1, (char*)rdac2Value);
        *loadRdac2Button = 0;
    }

    if(*shutdown1Button != shutdown1ButtonOld)  // If the Shutdown 1 button was pressed,
    {                                           // shutdown RDAC1.
        shutdown1ButtonOld = *shutdown1Button;
        if(*shutdown1Button == 1)
        {
            serialWord = 0x40;
            I2C_Write(XPAR_AXI_IIC_0_BASEADDR, AD5172_I2C_ADDRESS, -1, 1, (char*)&serialWord);
        }
        else
        {
            serialWord = 0x00;
            I2C_Write(XPAR_AXI_IIC_0_BASEADDR, AD5172_I2C_ADDRESS, -1, 1, (char*)&serialWord);
        }
    }

    if(*shutdown2Button != shutdown2ButtonOld)  // If the Shutdown 2 button was pressed,
    {                                           // shutdown RDAC2.
        shutdown2ButtonOld = *shutdown2Button;
        if(*shutdown2Button == 1)
        {
            serialWord = 0xC0;
            I2C_Write(XPAR_AXI_IIC_0_BASEADDR, AD5172_I2C_ADDRESS, -1, 1, (char*)&serialWord);
        }
        else
        {
            serialWord = 0x80;
            I2C_Write(XPAR_AXI_IIC_0_BASEADDR, AD5172_I2C_ADDRESS, -1, 1, (char*)&serialWord);
        }
    }

    if(*program1Button)     // If the Program 1 button was pressed,
    {                       // writes OTP memory 1.
        serialWord = 0x00;
        I2C_Write(XPAR_AXI_IIC_0_BASEADDR, AD5172_I2C_ADDRESS, -1, 1, (char*)&serialWord); //Select RDAC1

        //serialWord = 0x20;
        //I2C_Write(XPAR_AXI_IIC_0_BASEADDR, AD5172_I2C_ADDRESS, -1, 1, (char*)&serialWord); //Program OTP

        I2C_Read(XPAR_AXI_IIC_0_BASEADDR, AD5172_I2C_ADDRESS, -1, 2, rxBuf); // Read data.
        *fuse1Status = (rxBuf[1] >> 6);

        *program1Button = 0;
    }

    if(*program2Button)     // If the Program 2 button was pressed,
    {                       // writes OTP memory 2.
        serialWord = 0x80;
        I2C_Write(XPAR_AXI_IIC_0_BASEADDR, AD5172_I2C_ADDRESS, -1, 1, (char*)&serialWord); //Select RDAC2

        //serialWord = 0xA0;
        //I2C_Write(XPAR_AXI_IIC_0_BASEADDR, AD5172_I2C_ADDRESS, -1, 1, (char*)&serialWord); //Program OTP

        I2C_Read(XPAR_AXI_IIC_0_BASEADDR, AD5172_I2C_ADDRESS, -1, 2, rxBuf); // Read data.
        *fuse2Status = (rxBuf[1] >> 6);

        *program2Button = 0;
    }

    if(*sendDataButton)     // If the Send Data button was pressed,
    {                       // sends the value from the switches to the I2C.
        txBuf[0] = (*a0 << 7) + (*sdBit << 6) + (*tBit << 5) + (*zeroBit << 4) +
                   (*owBit << 3) + (*x1Bit << 2) + (*x2Bit << 1) + *x3Bit;
        if(*bytesButton)
        {
            txBuf[1] = (*d7Bit << 7) + (*d6Bit << 6) + (*d5Bit << 5) + (*d4Bit << 4) +
                       (*d3Bit << 3) + (*d2Bit << 2) + (*d1Bit << 1) + *d0Bit;
        }
        else
        {
            txBuf[1] = txBuf[0];
        }
        I2C_Write(XPAR_AXI_IIC_0_BASEADDR, AD5172_I2C_ADDRESS, -1, 2, txBuf);

        serialWord = 0x00;
        I2C_Write(XPAR_AXI_IIC_0_BASEADDR, AD5172_I2C_ADDRESS, -1, 1, (char*)&serialWord); //Select RDAC1
        I2C_Read(XPAR_AXI_IIC_0_BASEADDR, AD5172_I2C_ADDRESS, -1, 2, rxBuf); // Read data.
        *fuse1Status = (rxBuf[1] >> 6);

        serialWord = 0x80;
        I2C_Write(XPAR_AXI_IIC_0_BASEADDR, AD5172_I2C_ADDRESS, -1, 1, (char*)&serialWord); //Select RDAC2
        I2C_Read(XPAR_AXI_IIC_0_BASEADDR, AD5172_I2C_ADDRESS, -1, 2, rxBuf); // Read data.
        *fuse2Status = (rxBuf[1] >> 6);

        *sendDataButton = 0;
    }

    return(TRUE);
}
